/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * prj_rpt_dlg
 *
 * Synopsis:
 *   Gather the information needed to generate the basic project related
 *   reports, and then generate the appropriate report.
 *
 * $Id: prj_rpt_dlg.c,v 1.10 2002/01/20 20:48:16 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330 
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <errno.h>
#ifdef HAVE_STATBUF_H
#  include <statbuf.h>
#endif
#include <stdlib.h>
#include <sys/stat.h>
#include <syslog.h>
#include <unistd.h>

#include <gnome.h>
#include <libpq-fe.h>
#include <preps/change_log.h>
#include <preps/prj_rpts_html.h>

#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "prj_rpt_dlg.h"
#include "props.h"
#include "util.h"

/*
 * Data labels for data stored with the main widget.
 */
#define RPT_TYPE      "report_type"
#define PRJ_CLIST     "project_clist"
#define SHOW_ALL_CB   "show_all_cb"
#define SEL_ALL_BTN   "sel_all_btn"
#define UNSEL_ALL_BTN "unsel_all_btn"
#define PATH_ENTRY    "path_entry"
#define FILE_SEL_BTN  "file_sel_button"

/*
 * Only include projects that the current user has some sort of
 * access to.
 */
#define PK_POS      0
#define NAME_POS    1
#define ACTIVE_POS  2
#define PRJ_QUERY "\
SELECT distinct prj.project_num, prj.name, prj.active \
  FROM project prj \
 WHERE (prj.login_id = '%s' \
        OR exists (SELECT * \
                     FROM submitter s \
                    WHERE s.project_num = prj.project_num \
                      AND s.login_id = '%s') \
        OR exists (SELECT * \
                     FROM responsible r \
                    WHERE r.project_num = prj.project_num \
                      AND r.login_id = '%s')) "
#define PRJ_ACTIVE " AND prj.active = TRUE "
#define PRJ_ORDER_BY " ORDER BY prj.name "

#define DIR_MAX     2047

#define INVALID_DIR   -3
#define INVALID_FILE  -4

#define GEN_BUTTON     0
#define VIEW_BUTTON    1
#define CLOSE_BUTTON   2

typedef struct FileSelStruct
{
   GtkFileSelection  *file_sel_dlg;
   GtkEntry          *file_entry;
} FileSelStruct;


static void
on_file_sel_destroy (GtkWidget *widget, gpointer user_data)
{
   g_assert (user_data);
   g_free (user_data);
   gtk_main_quit ();
}


static void
on_file_sel_cancel (GtkButton *button, gpointer user_data)
{
   g_assert (user_data);
   gtk_widget_destroy
      (GTK_WIDGET (((FileSelStruct *)user_data)->file_sel_dlg));
}


static void
on_file_sel_ok (GtkButton *button, gpointer user_data)
{
   GtkFileSelection  *fsd;
   GtkEntry          *entry;
   GString           *file_name;

   g_assert (user_data);
   fsd = ((FileSelStruct *)user_data)->file_sel_dlg;
   entry = ((FileSelStruct *)user_data)->file_entry;

   file_name = g_string_new (gtk_file_selection_get_filename (fsd));
   
   gtk_entry_set_text (entry, file_name->str);
   
   gtk_widget_destroy (GTK_WIDGET (fsd));
   g_string_free (file_name, TRUE);
}


static void
draw_project_list (GtkWidget  *main_widget)
{
   PGconn           *conn;
   GtkCList         *clist;
   GtkToggleButton  *show_all_cb;
   gboolean          show_inactive;
   PGresult         *res;
   GString          *sql_buffer;
   gint              pk;
   gint              i, n;
   gchar            *list_buf[2];

   clist = GTK_CLIST (get_widget (main_widget, PRJ_CLIST));
   g_assert (clist != NULL);
   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   show_all_cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, SHOW_ALL_CB));
   g_assert (show_all_cb != NULL);

   show_inactive = gtk_toggle_button_get_active (show_all_cb);

   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, PRJ_QUERY, PQuser (conn),
                     PQuser (conn), PQuser (conn));
   if (!show_inactive)
      {
      sql_buffer = g_string_append (sql_buffer, PRJ_ACTIVE);
      }
   sql_buffer = g_string_append (sql_buffer, PRJ_ORDER_BY);

   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("filling project list on report dlg")))
      {
      n = PQntuples (res);
      gtk_clist_freeze (clist);
      gtk_clist_clear (clist);
      for (i = 0; i < n; i++)
         {
         list_buf[0] = g_malloc (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
         if (toupper ((PQgetvalue (res, i, ACTIVE_POS))[0]) == 'T')
            {
            list_buf[1] = g_malloc (strlen ("Yes") + 1);
            strcpy (list_buf[1], "Yes");
            }
         else
            {
            list_buf[1] = g_malloc (strlen ("No") + 1);
            strcpy (list_buf[1], "No");
            }
         gtk_clist_append (clist, list_buf);
         pk = atoi (PQgetvalue (res, i, PK_POS));
         gtk_clist_set_row_data (clist, i, GINT_TO_POINTER (pk));

         g_free (list_buf[0]);
         g_free (list_buf[1]);
         }
      gtk_clist_thaw (clist);
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   return;
}


static void
on_prj_rpt_dlg_show (GtkWidget  *widget,
                     gpointer    user_data)
{
   GtkToggleButton *cb;
   GtkEntry        *entry;
   GString         *shall_key;
   GString         *dirfile_key;
   gint             rpt_type;
   gboolean         is_def;
   gboolean         flag;
   gchar           *buffer;
   gchar            curr_work_dir[DIR_MAX + 1];

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                    RPT_TYPE)); 
   entry = GTK_ENTRY (get_widget (widget, PATH_ENTRY));
   g_assert (entry != NULL);
   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SHOW_ALL_CB));
   g_assert (cb != NULL);

   /*
    * Build the keys that we will need to check.
    */
   shall_key = g_string_new ("");
   dirfile_key = g_string_new ("");
   switch (rpt_type)
      {
      case StatusRpt:
         g_string_sprintf (shall_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           STAT_RPT_SEC, SHOW_ALL, "false");
         g_string_sprintf (dirfile_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           STAT_RPT_SEC, DIR_NAME, "bogus");
         break;
      case VersionRpt:
         g_string_sprintf (shall_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           VER_RPT_SEC, SHOW_ALL, "false");
         g_string_sprintf (dirfile_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           VER_RPT_SEC, DIR_NAME, "bogus");
         break;
      case ChangeLog:
         g_string_sprintf (shall_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           CHLG_RPT_SEC, SHOW_ALL, "false");
         g_string_sprintf (dirfile_key, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                           CHLG_RPT_SEC, DIR_NAME, "bogus");
         break;
      default:
         g_assert_not_reached();
      }

   /*
    * Set the show_all_cb appropriatly, then draw the project list.
    */
   flag = gnome_config_get_bool_with_default (shall_key->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);
   draw_project_list (widget);

   /*
    * Set the dir/file name entry to whatever was used last time, CWD if
    * there is no defined "last time".
    */
   buffer = gnome_config_get_string_with_default (dirfile_key->str, &is_def);
   if (!is_def)
      {
      gtk_entry_set_text (entry, buffer);
      }
   else if (getcwd (curr_work_dir, DIR_MAX))
      {
      gtk_entry_set_text (entry, curr_work_dir);
      }
   g_free (buffer);

   /*
    * general clean up
    */
   g_string_free (shall_key, TRUE);
   g_string_free (dirfile_key, TRUE);
}


static void
on_prj_rpt_dlg_destroy (GtkWidget  *widget,
                        gpointer    user_data)
{
   GtkToggleButton *cb;
   GtkEntry        *entry;
   GString         *shall_key;
   GString         *dirfile_key;
   GString         *filename;
   gint             rpt_type;

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                    RPT_TYPE)); 
   entry = GTK_ENTRY (get_widget (widget, PATH_ENTRY));
   g_assert (entry != NULL);
   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SHOW_ALL_CB));
   g_assert (cb != NULL);

   /*
    * build the keys for the data that we will be saving
    */
   shall_key = g_string_new ("");
   dirfile_key = g_string_new ("");
   switch (rpt_type)
      {
      case StatusRpt:
         g_string_sprintf (shall_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           STAT_RPT_SEC, SHOW_ALL);
         g_string_sprintf (dirfile_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           STAT_RPT_SEC, DIR_NAME);
         break;
      case VersionRpt:
         g_string_sprintf (shall_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           VER_RPT_SEC, SHOW_ALL);
         g_string_sprintf (dirfile_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           VER_RPT_SEC, DIR_NAME);
         break;
      case ChangeLog:
         g_string_sprintf (shall_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           CHLG_RPT_SEC, SHOW_ALL);
         g_string_sprintf (dirfile_key, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                           CHLG_RPT_SEC, FILE_NAME);
         break;
      default:
         g_assert_not_reached();
      }

   /*
    * Save the data, sync the file, and clean up.
    */
   gnome_config_set_bool (shall_key->str, gtk_toggle_button_get_active (cb));
   gnome_config_set_string (dirfile_key->str, gtk_entry_get_text (entry));

   filename = g_string_new ("");
   g_string_sprintf (filename, CONF_FILE, PACKAGE, DLG_INI_FILE);
   gnome_config_sync_file (filename->str);
   g_string_free (filename, TRUE);

   g_string_free (shall_key, TRUE);
   g_string_free (dirfile_key, TRUE);

   /*
    * Check the main widget, and see if we are running 
    * modally.  It is assumed that we were started via the
    * execute_xxx function in this case, so we need to quit
    * the main loop created there.
    */
   if (GTK_WINDOW (widget)->modal)
      {
      gtk_main_quit ();
      }

   return;
}


static void
on_show_all_cb_toggled (GtkToggleButton  *btn,
                        gpointer          user_data)
{
   draw_project_list (GTK_WIDGET (user_data));
}


static void
on_sel_all_button_clicked (GtkButton  *button,
                           gpointer    user_data)
{
   GtkCList   *clist;
   gint        i;

   clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), PRJ_CLIST));

   gtk_clist_freeze (clist);
   for (i = 0; i < clist->rows; i++)
      {
      gtk_clist_select_row (clist, i, 0);
      }
   gtk_clist_thaw (clist);
}


static void
on_unsel_all_button_clicked (GtkButton  *button,
                             gpointer    user_data)
{
   GtkCList   *clist;
   gint        i;

   clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), PRJ_CLIST));

   gtk_clist_freeze (clist);
   for (i = 0; i < clist->rows; i++)
      {
      gtk_clist_unselect_row (clist, i, 0);
      }
   gtk_clist_thaw (clist);
}


static void
on_file_sel_btn_clicked (GtkButton  *button,
                         gpointer    user_data)
{
   GtkWidget      *filewin;
   GtkEntry       *entry;
   FileSelStruct  *data;
   GString        *buffer;
   struct stat     stat_buf;
   PrjRptType      rpt_type;

   entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), PATH_ENTRY));
   g_assert (entry != NULL);

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (user_data),
                                                    RPT_TYPE)); 
   if (rpt_type != ChangeLog)
      {
      filewin = gtk_file_selection_new (_("Select Output Directory..."));
      /*
       * We only want to get directories.  Disable all "file" selection
       * stuff, and only allow for selecting directories.
       */
      gtk_widget_set_sensitive (GTK_FILE_SELECTION (filewin)->file_list,
                                FALSE);
      gtk_widget_set_sensitive (GTK_FILE_SELECTION (filewin)->selection_entry,
                                FALSE);
      }
   else
      {
      filewin = gtk_file_selection_new (_("Select Output File..."));
      }

   data = g_malloc (sizeof (FileSelStruct));
   data->file_sel_dlg = GTK_FILE_SELECTION (filewin);
   data->file_entry = entry;

   gtk_signal_connect (GTK_OBJECT (filewin), "destroy",
                       GTK_SIGNAL_FUNC (on_file_sel_destroy), data);
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filewin)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC (on_file_sel_ok), data);
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filewin)->cancel_button),
                       "clicked", GTK_SIGNAL_FUNC (on_file_sel_cancel), data);

   buffer = g_string_new (gtk_entry_get_text (entry));
   if (buffer->len > 0)
      {
      if (stat (buffer->str, &stat_buf) == 0)
         {
         if (S_ISDIR (stat_buf.st_mode))
            {
            if (buffer->str[buffer->len - 1] != '/')
               {
               buffer = g_string_append_c (buffer, '/');
               }
            gtk_file_selection_set_filename (GTK_FILE_SELECTION (filewin),
                                             buffer->str);
            }
         }
      }
   g_string_free (buffer, TRUE);

   gtk_widget_show (filewin);
   gtk_window_set_modal (GTK_WINDOW (filewin), TRUE);

   gtk_main ();
}


static void
set_view_button_status (GnomeDialog *dlg, const gchar *path)
{
   PrjRptType   rpt_type;
   GString     *file_name;
   struct stat  stat_buf;
   gboolean     btn_active = FALSE;

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                    RPT_TYPE)); 
   file_name = g_string_new (path);
   if (rpt_type != ChangeLog)
      {
      file_name = g_string_append (file_name, "/index.html");
      }

   if (stat (file_name->str, &stat_buf) == 0)
      {
      if (S_ISREG (stat_buf.st_mode))
         {
         btn_active = TRUE;
         }
      }
   gnome_dialog_set_sensitive (dlg, VIEW_BUTTON, btn_active);
   g_string_free (file_name, FALSE);
}


static void
on_close_button_clicked (GnomeDialog *dlg)
{
   gnome_dialog_close (dlg);
}


static void
on_gen_button_clicked (GnomeDialog  *dlg)
{
   struct stat       stat_buf;
   PGconn           *conn;
   GtkToggleButton  *show_all_cb;
   GtkCList         *prj_clist;
   GtkEntry         *dir_entry;
   GString          *path_name;
   PrjRptType        rpt_type;
   gint              rtn = -1;
   gint              row;
   gboolean          active_only;
   GList            *prj_list = NULL;
   GList            *list_iter;
   gint              status;
   GString          *msg;
   GtkWidget        *msg_dlg;

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                    RPT_TYPE)); 
   dir_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), PATH_ENTRY));
   g_assert (dir_entry);
   show_all_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (dlg),
                                                SHOW_ALL_CB));
   g_assert (show_all_cb);
   prj_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), PRJ_CLIST));
   g_assert (prj_clist);
   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);


   for (list_iter = g_list_first (prj_clist->selection);
        list_iter != NULL;
        list_iter = list_iter->next)
      {
      row = GPOINTER_TO_INT (list_iter->data);
      prj_list = g_list_append (prj_list,
                                gtk_clist_get_row_data (prj_clist, row));
      }
   prj_list = g_list_first (prj_list);
   if (!prj_list)
      {
      if (rpt_type == ChangeLog)
         {
         msg_dlg = gnome_message_box_new
            (_("You must select a project."),
             GNOME_MESSAGE_BOX_ERROR,
             GNOME_STOCK_BUTTON_OK,
             NULL);
         gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
         gnome_dialog_run (GNOME_DIALOG (msg_dlg));
         return;
         }
      else
         {
         msg_dlg = gnome_message_box_new
            (_("Generating the report without selecting any projects will\n"
               "result in the report being generated for all listed projects.\n"
               "Is this what you want?"),
             GNOME_MESSAGE_BOX_QUESTION,
             GNOME_STOCK_BUTTON_YES,
             GNOME_STOCK_BUTTON_NO,
             NULL);
         gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
         if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 1)
            {
            return;
            }
         }
      }

   path_name = g_string_new (gtk_entry_get_text (dir_entry));

   active_only = !gtk_toggle_button_get_active (show_all_cb);

   if (path_name->len > 0)
      {
      status = stat (path_name->str, &stat_buf);
      if (status == 0 || rpt_type == ChangeLog)
         {
         switch (rpt_type)
            {
            case StatusRpt:
               if (S_ISDIR (stat_buf.st_mode))
                  {
                  rtn = create_project_status_report (conn,
                                                      path_name->str,
                                                      prj_list,
                                                      active_only);
                  }
               else
                  {
                  rtn = INVALID_DIR;
                  }
               break;

            case VersionRpt:
               if (S_ISDIR (stat_buf.st_mode))
                  {
                  rtn = create_project_version_report (conn,
                                                       path_name->str,
                                                       prj_list,
                                                       active_only,
                                                       FALSE);
                  }
               else
                  {
                  rtn = INVALID_DIR;
                  }
               break;

            case ChangeLog:
               if (status == 0 &&
                   ((S_ISDIR (stat_buf.st_mode) ||
                     S_ISCHR (stat_buf.st_mode) ||
                     S_ISBLK (stat_buf.st_mode) ||
                     S_ISFIFO (stat_buf.st_mode)||
                     S_ISSOCK (stat_buf.st_mode))))
                  {
                  rtn = INVALID_FILE;
                  }
               else
                  {
                  rtn = create_change_log (conn,
                                           path_name->str, 
                                           GPOINTER_TO_INT (prj_list->data));
                  }
               break;

            default:
               g_assert_not_reached ();
            } /* end switch (rpt_type) */

         switch (rtn)
            {
            case 0:
               msg = g_string_new ("");
               g_string_sprintf
                  (msg,
                   _("The report was succuessfully generated in\n%s"),
                   path_name->str);
               msg_dlg = gnome_message_box_new (msg->str,
                                                GNOME_MESSAGE_BOX_INFO,
                                                GNOME_STOCK_BUTTON_OK,
                                                NULL);
               gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
               gnome_dialog_run (GNOME_DIALOG (msg_dlg));
               g_string_free (msg, TRUE);
               break;

            case -1:
               msg = g_string_new ("");
               g_string_sprintf
                  (msg,
                   _("An error occurred generating the report in\n%s\n"),
                   path_name->str);
               msg = g_string_append (msg, g_strerror (errno));
               msg_dlg = gnome_message_box_new (msg->str,
                                                GNOME_MESSAGE_BOX_ERROR,
                                                GNOME_STOCK_BUTTON_OK,
                                                NULL);
               gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
               gnome_dialog_run (GNOME_DIALOG (msg_dlg));
               g_string_free (msg, TRUE);
               break;

            case -2:
               msg = g_string_new ("");
               g_string_sprintf
                  (msg,
                   _("An error occurred generating the report in\n%s\n"),
                   path_name->str);
               msg = g_string_append
                  (msg,  
                   _("libPRepS internal SQL error - see syslog"));
               msg_dlg = gnome_message_box_new (msg->str,
                                                GNOME_MESSAGE_BOX_ERROR,
                                                GNOME_STOCK_BUTTON_OK,
                                                NULL);
               gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
               gnome_dialog_run (GNOME_DIALOG (msg_dlg));
               g_string_free (msg, TRUE);
               break;

            case INVALID_DIR:
               msg_dlg = gnome_message_box_new
                  (_("The specified output path is not a valid, existing directory."),
                   GNOME_MESSAGE_BOX_ERROR,
                   GNOME_STOCK_BUTTON_OK,
                   NULL);
               gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
               gnome_dialog_run (GNOME_DIALOG (msg_dlg));
               break;

            case INVALID_FILE:
               msg = g_string_new ("");
               g_string_sprintf
                  (msg,
                   _("%s\nis either a directory or a spcial type of file.\n"),
                   path_name->str);
               msg = g_string_append (msg, _("Please choose another name."));
               msg_dlg = gnome_message_box_new (msg->str,
                                                GNOME_MESSAGE_BOX_ERROR,
                                                GNOME_STOCK_BUTTON_OK,
                                                NULL);
               gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
               gnome_dialog_run (GNOME_DIALOG (msg_dlg));
               g_string_free (msg, TRUE);
               break;

            default:
               g_assert_not_reached ();
            } /* end switch (rtn) */
         }
      else /* stat() called failed */
         {
         msg = g_string_new (_("There is a problem with the path:\n"));
         msg = g_string_append (msg, path_name->str);
         msg = g_string_append_c (msg, '\n');
         msg = g_string_append (msg, g_strerror (errno));
         msg_dlg = gnome_message_box_new (msg->str,
                                          GNOME_MESSAGE_BOX_ERROR,
                                          GNOME_STOCK_BUTTON_OK,
                                          NULL);
         gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
         gnome_dialog_run (GNOME_DIALOG (msg_dlg));
         g_string_free (msg, TRUE);
         syslog (LOG_ERR, _("Failed to stat %s: %m"), path_name->str);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("You MUST supply an output path."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }

   g_list_free (prj_list);
   g_string_free (path_name, TRUE);
   set_view_button_status (dlg, gtk_entry_get_text (GTK_ENTRY (dir_entry)));

   return;
}


static void
on_path_entry_changed (GtkWidget *entry, gpointer user_data)
{
   set_view_button_status (GNOME_DIALOG (user_data),
                           gtk_entry_get_text (GTK_ENTRY (entry)));
}


static void
on_view_button_clicked (GnomeDialog  *dlg)
{
   gchar      *viewer;
   GString    *file_name;
   GtkEntry   *dir_entry;
   PrjRptType  rpt_type;

   rpt_type = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                    RPT_TYPE)); 
   dir_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), PATH_ENTRY));
   g_assert (dir_entry);

   file_name = g_string_new (gtk_entry_get_text (dir_entry));

   if (rpt_type == ChangeLog)
      {
      viewer = text_viewer();
      }
   else
      {
      viewer = html_viewer();
      file_name = g_string_append (file_name, "/index.html");
      }

   spawn_viewer (viewer, file_name->str);
   g_free (viewer);
   g_string_free (file_name, TRUE);

   return;
}


static void
on_prj_rpt_dlg_clicked (GnomeDialog *dlg,
                        gint         btn_cd,
                        gpointer     user_data)
{
   switch (btn_cd)
      {
      case GEN_BUTTON:
         on_gen_button_clicked (dlg);
         break;
      case VIEW_BUTTON:
         on_view_button_clicked (dlg);
         break;
      case CLOSE_BUTTON:
         on_close_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached ();
      }
}


static GtkWidget *
create_prj_rpt_dlg (GtkWidget  *parent,
                    PGconn     *conn,
                    PrjRptType  rpt_type)
{
   GtkWidget    *prj_rpt_dlg;
   GtkWidget    *dialog_vbox;
   GtkWidget    *file_sel_btn;
   GtkWidget    *hbox;
   GtkWidget    *hsep;
   GtkWidget    *label;
   GtkWidget    *path_entry;
   GtkWidget    *project_clist;
   GtkWidget    *scrollwin;
   GtkWidget    *sel_all_btn;
   GtkWidget    *show_all_cb;
   GtkWidget    *unsel_all_btn;
   GtkWidget    *vbox;
   GtkWidget    *popup;
   GtkTooltips  *tooltips;

   /*
    * Just use the parent's tooltips...
    */
   g_assert (parent);
   tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
   g_assert (tooltips);

   prj_rpt_dlg = gnome_dialog_new ("", NULL);
   gnome_dialog_set_parent (GNOME_DIALOG (prj_rpt_dlg), GTK_WINDOW (parent));
   gtk_container_border_width (GTK_CONTAINER (prj_rpt_dlg), 5);
   gtk_signal_connect (GTK_OBJECT (prj_rpt_dlg), "show",
                       GTK_SIGNAL_FUNC (on_prj_rpt_dlg_show),
                       NULL);
   gtk_signal_connect (GTK_OBJECT (prj_rpt_dlg), "destroy",
                       GTK_SIGNAL_FUNC (on_prj_rpt_dlg_destroy),
                       NULL);
   gtk_signal_connect (GTK_OBJECT (prj_rpt_dlg), "clicked",
                       GTK_SIGNAL_FUNC (on_prj_rpt_dlg_clicked),
                       NULL);
   switch (rpt_type)
      {
      case StatusRpt:
         gtk_window_set_title (GTK_WINDOW (prj_rpt_dlg), 
                               _("Generate the \"PRs by Status\" Report"));
         break;

      case VersionRpt:
         gtk_window_set_title
            (GTK_WINDOW (prj_rpt_dlg), 
             _("Generate the \"PRs by Project Version\" Report"));
         break;

      case ChangeLog:
         gtk_window_set_title (GTK_WINDOW (prj_rpt_dlg),
                               _("Generate Project ChangeLog"));
         break;

      default:
         g_assert_not_reached ();
      }
   gtk_window_set_policy (GTK_WINDOW (prj_rpt_dlg), FALSE, FALSE, TRUE);

   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), TOP_DOG,
                        prj_rpt_dlg);
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), MY_PARENT, parent);
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), MY_DB_CONN, conn);
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), RPT_TYPE, 
                        GINT_TO_POINTER (rpt_type));
   gnome_dialog_set_default (GNOME_DIALOG (prj_rpt_dlg), 0);
   gnome_dialog_set_close (GNOME_DIALOG (prj_rpt_dlg), FALSE);
   gnome_dialog_set_parent (GNOME_DIALOG (prj_rpt_dlg), GTK_WINDOW (parent));
   gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (prj_rpt_dlg),
                                           _("Generate"),
                                           GNOME_STOCK_PIXMAP_EXEC);
   gnome_dialog_append_button (GNOME_DIALOG (prj_rpt_dlg),
                               _("View"));
   gnome_dialog_append_button (GNOME_DIALOG (prj_rpt_dlg),
                               GNOME_STOCK_BUTTON_CLOSE);

   dialog_vbox = GNOME_DIALOG (prj_rpt_dlg)->vbox;
   gtk_box_set_spacing (GTK_BOX (dialog_vbox), 5);
   gtk_widget_show (dialog_vbox);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_widget_show (vbox);
   gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);

   label = gtk_label_new (_("Project List"));
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (scrollwin);
   gtk_box_pack_start (GTK_BOX (vbox), scrollwin, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);

   project_clist = gtk_clist_new (2);
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), PRJ_CLIST, project_clist);
   gtk_widget_show (project_clist);
   gtk_container_add (GTK_CONTAINER (scrollwin), project_clist);
   gtk_widget_set_usize (project_clist, 500, 150);
   gtk_clist_set_column_width (GTK_CLIST (project_clist), 0, 400);
   gtk_clist_column_titles_show (GTK_CLIST (project_clist));
   if (rpt_type == ChangeLog)
      {
      gtk_clist_set_selection_mode (GTK_CLIST (project_clist),
                                    GTK_SELECTION_SINGLE);
      }
   else
      {
      gtk_clist_set_selection_mode (GTK_CLIST (project_clist),
                                    GTK_SELECTION_MULTIPLE);
      }
   gtk_tooltips_set_tip
      (tooltips,
       project_clist,
       _("Select one or more project to be included in the report."),
       NULL);

   label = gtk_label_new ("Project");
   gtk_widget_show (label);
   gtk_clist_set_column_widget (GTK_CLIST (project_clist), 0, label);

   label = gtk_label_new ("Is Active?");
   gtk_widget_show (label);
   gtk_clist_set_column_widget (GTK_CLIST (project_clist), 1, label);

   show_all_cb = gtk_check_button_new_with_label (_("Show Inactive Projects"));
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), SHOW_ALL_CB, show_all_cb);
   gtk_widget_show (show_all_cb);
   gtk_box_pack_start (GTK_BOX (vbox), show_all_cb, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (show_all_cb), "toggled",
                       GTK_SIGNAL_FUNC (on_show_all_cb_toggled),
                       prj_rpt_dlg);
   gtk_tooltips_set_tip (tooltips,
                         show_all_cb,
                         _("Enable/Disable the display of inactive projects."),
                         NULL);

   if (rpt_type != ChangeLog)
      {
      hbox = gtk_hbox_new (FALSE, 5);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (dialog_vbox), hbox, TRUE, TRUE, 0);
   
      sel_all_btn = gtk_button_new_with_label (_("Select All"));
      gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg),
                           SEL_ALL_BTN,
                           sel_all_btn);
      gtk_widget_show (sel_all_btn);
      gtk_box_pack_start (GTK_BOX (hbox), sel_all_btn, TRUE, TRUE, 0);
      gtk_signal_connect (GTK_OBJECT (sel_all_btn), "clicked",
                       GTK_SIGNAL_FUNC (on_sel_all_button_clicked),
                       prj_rpt_dlg); 
      gtk_tooltips_set_tip (tooltips,
                            sel_all_btn,
                            _("Select all projects in the list."),
                            NULL);
   
      unsel_all_btn = gtk_button_new_with_label (_("Select None"));
      gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), 
                           UNSEL_ALL_BTN, unsel_all_btn);
      gtk_widget_show (unsel_all_btn);
      gtk_box_pack_start (GTK_BOX (hbox), unsel_all_btn, TRUE, TRUE, 0);
      gtk_signal_connect (GTK_OBJECT (unsel_all_btn), "clicked",
                          GTK_SIGNAL_FUNC (on_unsel_all_button_clicked),
                          prj_rpt_dlg); 
      gtk_tooltips_set_tip (tooltips,
                            unsel_all_btn,
                            _("Clear all current projects selections."),
                            NULL);
      }

   hsep = gtk_hseparator_new ();
   gtk_widget_show (hsep);
   gtk_box_pack_start (GTK_BOX (dialog_vbox), hsep, TRUE, TRUE, 0);

   vbox = gtk_vbox_new (FALSE, 0);
   gtk_widget_show (vbox);
   gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);

   if (rpt_type == ChangeLog)
      {
      label = gtk_label_new (_("ChangeLog Output File"));
      }
   else
      {
      label = gtk_label_new (_("Report Output Directory"));
      }
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

   hbox = gtk_hbox_new (FALSE, 5);
   gtk_widget_show (hbox);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

   path_entry = gtk_entry_new ();
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), PATH_ENTRY, path_entry);
   gtk_widget_show (path_entry);
   gtk_box_pack_start (GTK_BOX (hbox), path_entry, TRUE, TRUE, 0);
   gtk_tooltips_set_tip
      (tooltips,
       path_entry,
       _("Enter the path where the report should be created."),
       NULL);
   popup = attach_generic_edit_popup_menu (path_entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (prj_rpt_dlg), "popup1", popup,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (path_entry), "changed",
                       GTK_SIGNAL_FUNC (on_path_entry_changed),
                       prj_rpt_dlg); 

   file_sel_btn = gtk_button_new_with_label (" ... ");
   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), FILE_SEL_BTN, file_sel_btn);
   gtk_widget_show (file_sel_btn);
   gtk_box_pack_start (GTK_BOX (hbox), file_sel_btn, FALSE, FALSE, 0);
   gtk_tooltips_set_tip (tooltips,
                         file_sel_btn,
                         _("Press to select the report output directory."),
                         NULL);
   gtk_signal_connect (GTK_OBJECT (file_sel_btn), "clicked",
                       GTK_SIGNAL_FUNC (on_file_sel_btn_clicked),
                       prj_rpt_dlg);

   gtk_object_set_data (GTK_OBJECT (prj_rpt_dlg), TOOLTIPS, tooltips);

   return prj_rpt_dlg;
}


gint
execute_prj_rpt_dlg (GtkWidget  *parent,
                     PGconn     *conn,
                     PrjRptType  rpt_type)
{
   GtkWidget  *dlg;

   dlg = create_prj_rpt_dlg (parent, conn, rpt_type);

   gtk_widget_show (dlg);
   gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);

   gtk_main ();

   return 0;
}
